C
C     FUNCTION TO OUTPUT A1(W) AND A2(W) FITS TO RSS DATA
C     O. A. RONDON 10/2005
C     INPUTS:
C     W [GEV], Q**2 [GEV**2], ENERGY LOSS NU [GEV]
C     N = WHICH ASYMMETRY: 1 = A1, 2 = A2
C     I = NUMBER OF B-W RESONANCES: 3 OR 4
C     J = ORDER OF DIS BACKGROUND POLYNOMIAL: 2 OR 3
C     ONLY VALID N,I,J COMBINATIONS ARE:
C     A1 = 1,4,4
C     A2 = 2,4,4.
C               
C     LOGICAL ERR CAN BE USED TO SKIP W VALUES BELOW PI0 THRESHOLD

      REAL*8 FUNCTION A1A2FIT(W,QSQ,N,IMAX,JMAX,D,R2,R3,R4,DIS,ERR)
      IMPLICIT NONE
      INTEGER NUM_TERMS
      PARAMETER(NUM_TERMS=17)
      REAL*8 W, AR(4,4), WR(4,4), GR(4,4), B(4,4), A(4)
      REAL*8 BW(4), SUMBW, DIS
      INTEGER I,IMAX,J,JMAX,K,N
      REAL*8 D,R2,R3,R4
      REAL*8 XB,ENU,QSQ,M,MPI
      REAL*8 QCM,QCR(4),G(4),GG(4)
      REAL*8 KCM,KCR(4)
      INTEGER LR(4),JR(4)
      REAL*8 XR(4)
      LOGICAL ERR

      !B-W PARAMETERS: AR = AMPLITUDE, WR = CENTROID, GR = WIDTH
      !B : DIS BACKGROUND = X**A*(B0+(B1+B2*X+B3*X**2)*X)
      !A : EXPONENT
      REAL*8 PARS1(NUM_TERMS),PARS1_INIT(NUM_TERMS)
      REAL*8 PARS2(NUM_TERMS),PARS2_INIT(NUM_TERMS)
      COMMON/FITPARS/AR,WR,GR,B,A,PARS1,PARS2,PARS1_INIT,PARS2_INIT


C      DATA AR/     -0.5166,      0.2384,      0.4132,      0.1818,
C     &             -0.1172,      0.1880,      0.1630,      0.1498,
C     &              0,           0,           0,           0,
C     &              0,           0,           0,           0        /
C
C
C      DATA WR/      1.2262,      1.3498,      1.5536,      1.7393,
C     &              1.2265,      1.3447,      1.4881,      1.6753,
C     &              0,           0,           0,           0,
C     &              0,           0,           0,           0        /
C
C      DATA GR/      0.2412,      0.0586,      0.2901,      0.1277,
C     &              0.1589,      0.0985,      0.1112,      0.1875,
C     &              0,           0,           0,           0,
C     &              0,           0,           0,           0        /
C
C
C      DATA B /     0.13455,     0.22084,     0.10051,    -0.09316,
C     &             0.08879,     0.07794,    -0.13864,    -0.08338,
C     &             0,           0,           0,           0,
C     &             0,           0,           0,           0        /
C
C      DATA A/-0.18062,0.43254,   0, 0/

      !-------------------------------------------------------
     
     
      DATA LR/ 1, 2, 3, 3/
      DATA JR/ 1, 1, 2, 2/

      DATA XR/ 0.160D0, 3*0.350D0/

      PARAMETER ( M   = 0.93828D0 )
      PARAMETER ( MPI = 0.134D0   )


      !CHECK PI0 THRESHOLD
      ERR = .FALSE.

      
      IF(W.LT.(M+MPI)) THEN
        ERR = .TRUE.
        RETURN
      END IF
      !WHICH FIT?
 
      IF(N.EQ.1) THEN
        K = 1                      !      ONLY FIT FOR A1
      ELSE                         !      A2 FITS
        IF(IMAX.EQ.3) THEN
          K = 3                    !      3 RESONANCES + QUAD DIS
        ELSE
          IF(JMAX.EQ.3) THEN
            K = 4                  !      4 RESONANCES + QUAD DIS
          ELSE
            K = 2                  !      4 RESONANCES + CUBIC DIS
          ENDIF                    !      END POLY ORDER
        ENDIF                      !      END 3 OR 4 B-W
      ENDIF                        !      END A1 OR A2                      

      !SUM OF B-W

      QCM    = SQRT(((W**2+M**2-MPI**2)/2.D0/W)**2 - M**2)
      KCM    = SQRT(((W**2+M**2+QSQ)/2.D0/W)**2 - M**2)

      SUMBW  = 0.D0

      DO I=1,IMAX
        QCR(I) = SQRT(((WR(I,K)**2+M**2-MPI**2)/2.D0/WR(I,K))**2 - M**2)
        KCR(I) = SQRT(((WR(I,K)**2+M**2+QSQ)/2.D0/WR(I,K))**2 - M**2)
        G(I)   = GR(I,K)*(QCM/QCR(I))**(2*LR(I)+1)*
     &           ((QCR(I)**2+XR(I)**2)/(QCM**2+XR(I)**2))**LR(I)
        GG(I)  = GR(I,K)*(KCM/KCR(I))**(2*JR(I))*
     &          ((KCR(I)**2+XR(I)**2)/(KCM**2+XR(I)**2))**JR(I)
        BW(I)  = AR(I,K)*(KCR(I)/KCM)**2*WR(I,K)**2*G(I)*GG(I)
        BW(I)  = BW(I)/((WR(I,K)**2-W**2)**2+(WR(I,K)*G(I))**2)
        SUMBW  = SUMBW + BW(I)
      ENDDO


      !SAVE BW RESULTS

      D  = BW(1)
      R2 = BW(2)
      R3 = BW(3)
      R4 = BW(4)
      
      !DIS POLY

      !XB  = QSQ/(2.D0*M*ENU)
      !WRITE(66,*) XB
      XB = ((W**2-M**2)/QSQ + 1.0)**(-1)
      !WRITE(67,*) XB
      DIS = XB**A(K)*(B(1,K) + XB*(B(2,K) + B(3,K)*XB + B(4,K)*XB**2))


      IF(N.EQ.2) THEN              ! ONLY FOR A2 = 1/Q DEPENDENCE
        IF(QSQ.NE.0.D0) THEN
          DIS = DIS/SQRT(QSQ)
        ELSE
          DIS = 0.D0
        END IF
      ENDIF

      !A1 OR A2

      A1A2FIT = SUMBW + DIS
      RETURN
      END
 
